// Copyright 1996-2000 Omni Development, Inc.  All rights reserved.
//
// This software may only be used and reproduced according to the
// terms in the file OmniSourceLicense.html, which should be
// distributed with this project and can also be found at
// http://www.omnigroup.com/DeveloperResources/OmniSourceLicense.html.

#import "CDDBDatabase+OmniCDExtensions.h"

#import <OmniSCSI/OSCSICDProtocols.h>
#import <OmniSCSI/OSCSICDTableOfContents.h>


@implementation CDDBDatabase (OmniCDExtensions)

//
// OmniCDDatabase protocol
//

- (id) queryDatabase: (OSCSICDTableOfContents *) toc;
{
    id results;

    NS_DURING {
        // There can only be 99 tracks on a CD and we need N+1 entries.  There should
        // really be a define for this in OmniSCSI.
        CDDBTableOfContentsEntry  entries[100];
        unsigned int              entryCount, entryIndex, durationSeconds;
        OSCSILogicalBlockAddress  lba;
        OSCSIMinuteSecondsFrames  msf;
        
        entryCount = [toc numberOfTracks];
        for (entryIndex = 0; entryIndex < entryCount; entryIndex++) {
            lba = [toc startingLBAForTrackNumber: entryIndex + 1];
            msf = OSCSIAbsoluteLogicalBlockAddressToMinutesSecondsFrames(lba);

            entries[entryIndex].logicalBlockAddress = lba;
            entries[entryIndex].minutes = msf.minutes;
            entries[entryIndex].seconds = msf.seconds;
        }

        lba = [toc startingLBAForTrackNumber: OSCSI_LEAD_OUT_TRACK];
        msf = OSCSIAbsoluteLogicalBlockAddressToMinutesSecondsFrames(lba);
        entries[entryCount].logicalBlockAddress = lba;
        entries[entryCount].minutes = msf.minutes;
        entries[entryCount].seconds = msf.seconds;
        durationSeconds = msf.minutes * 60 + msf.seconds;
        
        [self connect];
        results = [self sendQueryForEntries: entries entryCount: entryCount durationSeconds: durationSeconds];

#warning Closing due to CDDB whackness.
        // If you do a 'read' on the same disc twice in the same connection, they give
        // you can error like "502 Already read entry: rock 8f09aa0a".  They apparently
        // either want you to reconnect for each request (the most likely since they
        // expire in 800 seconds anyway) or they want you to cache stuff for a little
        // while.  Later, we will do caching on a larger scale, but we'll probably want
        // to send the CDDB reqeust anyway to let them collect the stats.
        [self disconnect];
    } NS_HANDLER {
        results = localException;
    } NS_ENDHANDLER;

    return results;
}

@end

