// Copyright 1996-2000 Omni Development, Inc.  All rights reserved.
//
// This software may only be used and reproduced according to the
// terms in the file OmniSourceLicense.html, which should be
// distributed with this project and can also be found at
// http://www.omnigroup.com/DeveloperResources/OmniSourceLicense.html.

#import "OmniCDController.h"

#import "OmniCDUnitController.h"
#import <OmniSCSI/OmniSCSI.h>
#import <OmniAppKit/OAPreferenceController.h>

@implementation OmniCDController

- init;
{
    if (!(self = [super init]))
        return nil;

    unitControllerByTargetNumber = [[NSMutableDictionary alloc] init];

    return self;
}

- (IBAction) showPreferences: (id) sender;
{
    [[OAPreferenceController sharedPreferenceController] showPreferencesPanel: sender];
}

//
// NSApplication delegate
//

- (void) applicationDidFinishLaunching: (NSNotification *) note;
{
    if (geteuid()) {
        NSRunAlertPanel(nil,
                        @"This application must be run as the superuser.  Please set the 'setuid' bit on the executable.",
                        @"Abort", nil, nil);
        exit(1);
    }
}

- (BOOL)application:(NSApplication *)sender openFile:(NSString *)filename;
{
    OmniCDUnitController   *unitController;
    OSCSITargetNumberType   targetNumber;
    NSString               *contents, *mountPoint, *rawDeviceName, *targetNumberString;
    NSArray                *components;
    NSWorkspace            *workspace;

    // Under CR1, Workspace will put an icon on the desktop when removable media is mounted.
    // There doesn't seem to be any way to signal from your file system bundle that you
    // aren't going to mount the media as an actual filesystem.  We'll tell Workspace to
    // check again, which will remove the icon.  Unfortunately, it will flash on screen briefly.
    // We'll do this after a delay to make sure that we don't deadlock on NSWorkspace (since
    // it is calling us) and so that we'll be more sure that OmniCDAudio has deleted the
    // directory.  Once we have a VFS module that does CDDA access, this won't be necessary
    // since we'll be mounting it as a real file system as well.
    workspace = [NSWorkspace sharedWorkspace];
    [workspace performSelector: @selector(noteFileSystemChanged)
                    withObject: nil
                    afterDelay: 0.0];

    if (!(contents = [[NSString alloc] initWithContentsOfFile:filename])) {
        NSLog(@"couldn't open file");
        NSRunAlertPanel(nil, @"Error opening %@", @"Abort", nil, nil, filename);
        return NO;
    }

    components = [contents componentsSeparatedByString: @" "];
    targetNumberString = [components objectAtIndex: 0];
#warning This is a bad cast...
    targetNumber = [targetNumberString intValue];
    mountPoint = [components objectAtIndex: 1];
    
    // For some reason the following line crashes.
    //[contents release];

    OBASSERT(unitControllerByTargetNumber);
    rawDeviceName = [NSString stringWithFormat: @"/dev/rsd%dh", (int)targetNumber];
    unitController = [unitControllerByTargetNumber objectForKey: rawDeviceName];
    if (!unitController) {
        NS_DURING {
            OSCSIUnit   *unit;
            OSCSITarget *target;

            // This means that we will only use the 'rsd' interface instead of the 'sg'
            // interface.  For CD audio command, this seems to work fine.  There are
            // some SCSI-3 MMC commands that don't seem to work via the 'rsd' interface
            // but we aren't using them in this app :)
            // Later we *could* switch back to using the 'sg' interface, but that means
            // that we have to scan the bus.  Currently, if there were multiple ATAPI drives
            // that would cause you to have to insert media in all of the once that didn't have
            // media in them (the one we are opening presumably does have media).
            target = [[OSCSITarget alloc] initWithBusNumber: OSCSI_INVALID_BUS_NUMBER
                                               targetNumber: OSCSI_INVALID_TARGET_NUMBER
                                              rawDeviceName: rawDeviceName];
            
            unit = [target unitWithUnitNumber: 0];

            if (![unit conformsToProtocol: @protocol(OSCSICDRomAudioControl)]) {
                [target release];
                NSRunAlertPanel(nil, @"This device/driver combination does not support CD audio.", @"Abort", nil, nil);
                NS_VALUERETURN(NO, BOOL);
            }

            unitController = [[OmniCDUnitController alloc] initWithTarget: target];
            [target release];

            [unitControllerByTargetNumber setObject: unitController forKey: rawDeviceName];
            [unitController release];
        } NS_HANDLER {
            NSLog(@"Error while probing: %@", localException);
            return NO;
        } NS_ENDHANDLER;
    }

    [unitController mediaMountedAtPath: mountPoint];
    return YES;
}

- (void) unitControllerClosedForTarget: (OSCSITarget *) target;
{
    NSString *key;

    key = [target rawDeviceName];
    OBASSERT([unitControllerByTargetNumber objectForKey: key]);
    [unitControllerByTargetNumber removeObjectForKey: key];
}

@end

