// Copyright 1996-2000 Omni Development, Inc.  All rights reserved.
//
// This software may only be used and reproduced according to the
// terms in the file OmniSourceLicense.html, which should be
// distributed with this project and can also be found at
// http://www.omnigroup.com/DeveloperResources/OmniSourceLicense.html.
//
// $Header: /Network/Source/CVS/OmniGroup/./Applications/OmniCD/OmniCD/OmniCDUnitController.h,v 1.9 2000/09/12 19:13:38 bungi Exp $

#import <AppKit/AppKit.h>
#import <OmniFoundation/OmniFoundation.h>
#import <OmniSCSI/OSCSICDProtocols.h>

#import "OmniCDDatabase.h"

@class OSCSITarget;

typedef enum _OmniCDTimeDisplayFormat {
    OmniCDElapsedSinceStartOfCurrentTrackTimeDisplayFormat,
    OmniCDRemainingInCurrentTrackTimeDisplayFormat,
    OmniCDTotalInCurrentTrackTimeDisplayFormat,
    OmniCDElapsedSinceStartOfDiscTimeFormat,
    OmniCDRemainingOnDiscTimeDisplayFormat,
    OmniCDTotalOnDiscTimeDisplayFormat,
    //
    OmniCDLastTimeDisplayFormat
} OmniCDTimeDisplayFormat;


@interface OmniCDUnitController : OFObject
{
    IBOutlet NSWindow    *window;
    IBOutlet NSMatrix    *trackMatrix;
    IBOutlet NSButton    *playButton;
    IBOutlet NSButton    *pauseButton;
    IBOutlet NSButton    *stopButton;
    IBOutlet NSButton    *forwardButton;
    IBOutlet NSButton    *backwardButton;
    IBOutlet NSButton    *unloadButton;
    IBOutlet NSSlider    *volumeSlider;
    IBOutlet NSButton    *currentTimeButton;
    IBOutlet NSTextField *currentTrackTextField;

    NSString                              *_mountPoint;
    OSCSITarget                           *_target;
    id <NSObject, OSCSICDRomAudioControl>  _audioUnit;
    
    BOOL                                  _mediaPresent;
    BOOL                                  _supportsIdentification;
    BOOL                                  _supportsVolumeControl;
    BOOL                                  _supportsScan;

    NSConnection                         *_unitConnection;
    unsigned int                          _requestCounter;
    id <OmniCDDatabase>                   _database;
    NSDictionary                         *_discInfo;
    
    NSTimer                              *_updateTimer;

    OmniCDTimeDisplayFormat               _timeDisplayFormat;
    BOOL                                  _changeTrackOnPlay;
    unsigned int                          _selectedTrack;
    unsigned int                          _currentLBA;
}

- initWithTarget: (OSCSITarget *) target;

- (void) mediaMountedAtPath: (NSString *) mountPoint;

- (IBAction) play: (id) sender;
- (IBAction) pause: (id) sender;
- (IBAction) stopPlaying: (id) sender;
- (IBAction) forward: (id) sender;
- (IBAction) backward: (id) sender;
- (IBAction) setTrack: (id) sender;
- (IBAction) setVolume: (id) sender;
- (IBAction) eject: (id) sender;

- (IBAction) queryDatabase: (id) sender;

- (IBAction) changeTimeDisplayFormat: (id) sender;

@end
